*+--------------------------------------------------------------------
*+
*+ Source Module => c:\hmg.3.4.4\0\BACKUP\\TOOLS.PRG
*+
*+    Copyright(C) 1983-2021 by Auge & Ohr
*+
*+    Functions: Function VAR2CHAR()
*+               Function GetDrives()
*+               Function Search4Outlook()
*+               Function GetSetPST()
*+               Function ChangeDriveLetter()
*+               Function SetAttribut()
*+               Procedure SnapToCtrl()
*+
*+    Reformatted by Click! 2.05.38 on Aug-26-2021 at  2:13 am
*+
*+--------------------------------------------------------------------

#include "hmg.ch"
#include "Directry.ch"

#include "BACKUP.CH"

*+--------------------------------------------------------------------
*+
*+    Function VAR2CHAR()
*+
*+    Called from ( shellop.prg )   1 - static procedure error_shell()
*+                ( usbeject.prg )   3 - function ejectmedia()
*+
*+--------------------------------------------------------------------
*+
FUNCTION VAR2CHAR( cIn )

LOCAL cOut := hb_valToExp( cIn )
RETURN STRTRAN( cOut, '"', '' )

*+--------------------------------------------------------------------
*+
*+    Function GetDrives()
*+
*+    Called from ( backup.prg )   1 - static procedure fillcombo()
*+
*+--------------------------------------------------------------------
*+
FUNCTION GetDrives()

LOCAL n, cDrv, nDrv, cVolume := "", aDrive := {}

   FOR n := 1 TO 26
      cDrv := CHR( 64 + n )
      nDrv := GetDriveType( cDrv + ":\" + CHR( 0 ) )
      IF nDrv > 1
         IF nDrv == 2 .AND. UPPER( cDrv ) == "A"
            cVolume := [3 1/2"]
         ELSE
            cVolume := ""
            GetVolumeInformation( cDrv + ":\", @cVolume )
         ENDIF
         AADD( aDrive, { IF( n = 1, 1, nDrv ), cDrv, IF( EMPTY( cVolume ), "none", cVolume ) } )
      ENDIF
   NEXT
RETURN aDrive

*+--------------------------------------------------------------------
*+
*+    Function Search4Outlook()
*+
*+    Called from ( backup.prg )   1 - procedure main()
*+
*+--------------------------------------------------------------------
*+
FUNCTION Search4Outlook()

LOCAL RETVAR   := .F.
LOCAL lStatus  := .F.
LOCAL aTemp    := {}
LOCAL oReg
LOCAL aSubkeys
LOCAL i, iMax
LOCAL cfile
LOCAL nPosi
LOCAL aDir
LOCAL cKey
LOCAL cStart   := "HKEY_CURRENT_USER\Software\Microsoft\Office\"

   FOR i = 8 TO 99
      cKey := cStart + STRZERO( i, 2 ) + ".0\Outlook\Catalog"
      oReg := XbpReg() :NEW( cKey )
      IF oReg:Status()
         lStatus := .T.
         EXIT
      ENDIF
   NEXT

   IF lStatus = .T.
      oReg:ReadBinType( "C" )
      aSubkeys := oReg:ValueList( .T. )
      iMax := LEN( aSubkeys )
      FOR i := 1 TO iMax
         cfile := aSubkeys[ i ] [ 1 ]
         GetSetPST( cfile )
         RETVAR := .T.
      NEXT
   ENDIF

RETURN RETVAR

*+--------------------------------------------------------------------
*+
*+    Function GetSetPST()
*+
*+    Called from ( backup.prg )   1 - static procedure doinit()
*+                                   1 - static procedure donow()
*+                ( tools.prg )   1 - function search4outlook()
*+
*+--------------------------------------------------------------------
*+
FUNCTION GetSetPST( xValue )

STATIC cPST := ""
   IF PCOUNT() > 0
      cPST := xValue
   ENDIF
RETURN cPST

*+--------------------------------------------------------------------
*+
*+    Function ChangeDriveLetter()
*+
*+    Called from ( backup.prg )   1 - static function shthread()
*+
*+--------------------------------------------------------------------
*+
FUNCTION ChangeDriveLetter( cSourceDir, cPathTarget )

LOCAL cRet := ""
   cRet := SUBSTR( cPathTarget, 1, 1 )
   cRet += SUBSTR( cSourceDir, 2 )
RETURN cRet

*+--------------------------------------------------------------------
*+
*+    Function SetAttribut()
*+
*+    Called from ( shellop.prg )   1 - static procedure reset_attribut()
*+
*+--------------------------------------------------------------------
*+
FUNCTION SetAttribut( cFileName, lArchive, lReadOnly, lHidden, lSystem, lChangeDT, cDate, cTime )

LOCAL nAttribute := 0, nError, dDate := CTOD( cDate )

   IF lArchive
      nAttribute += FILE_ATTRIBUTE_ARCHIVE
   ELSEIF lReadOnly
      nAttribute += FILE_ATTRIBUTE_READONLY
   ELSEIF lHidden
      nAttribute += FILE_ATTRIBUTE_HIDDEN
   ELSEIF lSystem
      nAttribute += FILE_ATTRIBUTE_SYSTEM
   ENDIF

   IF lChangeDT
      IF !hb_FSetDateTime( cFileName, dDate, cTime )
         MsgInfo( "Invalid date/time for " + cFileName, "Error SetAttribut" )
      ENDIF
   ENDIF

   nError := SetFAttr( cFileName, nAttribute )
   IF nError <> 0
      DO CASE
         CASE nError = ER_FILE_NOT_FOUND
            MsgInfo( "File not found " + cFileName, "Error SetAttribut" )
         CASE nError = ER_PATH_NOT_FOUND
            MsgInfo( "Path not found " + cFileName, "Error SetAttribut" )
         CASE nError = ER_ACCESS_DENIED
            MsgInfo( "Access denied to file " + cFileName, "Error SetAttribut" )
      ENDCASE
   ENDIF

RETURN Nil

*+--------------------------------------------------------------------
*+
*+    Procedure SnapToCtrl()
*+
*+    Called from ( backup.prg )   1 - static procedure doinit()
*+                                   1 - static procedure open_combo()
*+
*+--------------------------------------------------------------------
*+
PROCEDURE SnapToCtrl( cForm, cControl )

LOCAL hWnd, nRowCtrl, nColCtrl, nHeightCtrl, nWidthCtrl, nColPointer, nRowPointer

   IF !( IsWindowActive( &cForm ) ) .OR. !( IsControlDefined( &cControl, &cForm ) )
      RETURN
   ENDIF

   hWnd := GetProperty( cForm, 'Handle' )

   nRowCtrl := GetProperty( cForm, cControl, 'Row' )
   nColCtrl := GetProperty( cForm, cControl, 'Col' )
   nHeightCtrl := GetProperty( cForm, cControl, 'Height' )
   nWidthCtrl := GetProperty( cForm, cControl, 'Width' )

   //Center of control
   nRowPointer := nRowCtrl + ( nHeightCtrl / 2 )
   nColPointer := nColCtrl + ( nWidthCtrl / 2 )

   HMG_SetCursorPos( hWnd, nRowPointer, nColPointer )
   DoMethod( cForm, cControl, "Setfocus" )
RETURN

*+ EOF: TOOLS.PRG
