*+--------------------------------------------------------------------
*+
*+ Source Module => c:\hmg.3.4.4\0\BACKUP\\USBEJECT.PRG
*+
*+    Copyright(C) 1983-2021 by Auge & Ohr
*+
*+    Functions: Procedure Do_EjectForm()
*+               Static Procedure DoRunOnce()
*+               Function EjectMedia()
*+               Static Function CreateVolHandle()
*+               Static Function LockVolume()
*+               Static Function DismountVolume()
*+               Static Function AutoEjectVolume()
*+               Static Function UnlockVolume()
*+               Static Function CloseVolume()
*+               Static Function PreventRemovalOfVolume()
*+               Static Procedure SayInfo()
*+
*+    Reformatted by Click! 2.05.38 on Aug-26-2021 at  2:13 am
*+
*+--------------------------------------------------------------------

#include "hmg.ch"
#include "COMMON.CH"
#include "DLL.ch"

#include "BACKUP.CH"

STATIC cEject := ""

MEMVAR _HMG_SYSDATA

*+--------------------------------------------------------------------
*+
*+    Procedure Do_EjectForm()
*+
*+    Called from ( backup.prg )   1 - procedure doeject()
*+
*+--------------------------------------------------------------------
*+
PROCEDURE Do_EjectForm( cDrive, UseCDdrives )

   DEFINE WINDOW EjectDlg ;
         AT 0, 0 ;
         WIDTH 442 ;
         HEIGHT 479 ;
         TITLE "Eject " ;
         ICON "A1MAIN" ;
         NOMAXIMIZE ;
         NOMINIMIZE ;
         NOSIZE ;
         ON RELEASE Nil ;
         BACKCOLOR { 8, 8, 8 }

      DEFINE LABEL EjectInfo
         ROW 20
         COL 20
         WIDTH 420
         HEIGHT 450
         FONTNAME "Arial"
         FONTSIZE 20
         BACKCOLOR { 8, 8, 8 }
         FONTCOLOR { 0, 255, 255 }
         VISIBLE .T.
      END LABEL

      DEFINE TIMER internal_timer_4 INTERVAL 100 ACTION DoRunOnce( cDrive, UseCDdrives )
   END WINDOW

   CENTER WINDOW EjectDlg
   Activate Window EjectDlg

RETURN

*+--------------------------------------------------------------------
*+
*+    Static Procedure DoRunOnce()
*+
*+    Called from ( usbeject.prg )   1 - procedure do_ejectform()
*+
*+--------------------------------------------------------------------
*+
STATIC PROCEDURE DoRunOnce( cDrive, UseCDdrives )

   DoMethod( "EjectDlg", "internal_timer_4", "RELEASE" )
   DoMethod( "EjectDlg", "Setfocus" )

   EjectMedia( cDrive, UseCDdrives )

   EjectDlg.Release

RETURN

*+--------------------------------------------------------------------
*+
*+    Function EjectMedia()
*+
*+    Called from ( usbeject.prg )   1 - static procedure dorunonce()
*+
*+--------------------------------------------------------------------
*+
FUNCTION EjectMedia( cDrive, UseCDdrives )

   /****************************************************************
*
* USBeject based on https://support.microsoft.com/en-us/kb/165721
*
* will eject CD / DVD and "flush Buffer" if USB Drive
*
****************************************************************/

LOCAL hVolRead  := - 1                                                // "CreateFileA"
LOCAL nLock     := 0                                                  // "DeviceIoControl",hVolRead, FSCTL_LOCK_VOLUME
LOCAL nDisMount := 0                                                  // "DeviceIoControl",hVolRead, FSCTL_DISMOUNT_VOLUME
LOCAL nEject    := 0                                                  // "DeviceIoControl",hVolRead, IOCTL_STORAGE_EJECT_MEDIA
LOCAL nAccess   := 0
LOCAL nType     := 0
LOCAL cMsg      := ""
LOCAL nRemove   := 0
LOCAL lRet      := .T.
LOCAL cAction   := GETENV( "SYSTEMROOT" ) + "\SYSTEM32\HOTPLUG.DLL"
LOCAL cRoot     := "RUNDLL32.EXE "
LOCAL cPath     := GETENV( "TEMP" ) + "\"

   DEFAULT UseCDdrives TO .F.

   IF PCOUNT() = 0
      MsgInfo( "need Drive Letter" )
      RETURN .F.
   ENDIF

   cEject := ""

   nType := DriveType( cDrive )
   DO CASE
      CASE nType = DRIVE_UNKNOWN
      CASE nType = DRIVE_NO_ROOT_DIR
      CASE nType = DRIVE_REMOVABLE                                    // Floppy
         nAccess := GENERIC_READ + GENERIC_WRITE
      CASE nType = DRIVE_FIXED
      CASE nType = DRIVE_REMOTE
      CASE nType = DRIVE_CDROM
         IF UseCDdrives = .T.
            nAccess := GENERIC_READ
         ENDIF
      CASE nType = DRIVE_RAMDISK
      OTHERWISE
         MsgInfo( "can not use Type " + STR( nType ) )
   ENDCASE

   IF EMPTY( nAccess )
      RETURN .F.
   ENDIF

   SayInfo( "waiting for Drive " + cDrive + CRLF )
   hVolRead := CreateVolHandle( @cMsg, nType, nAccess, cDrive )

   IF hVolRead = - 1
      MsgInfo( "Unable to open drive " + cDrive )
      lRet := .F.
   ELSE

      /**********************************************************
      *
      * https://msdn.microsoft.com/en-us/library/Aa363216.aspx
      *
      * BOOL WINAPI DeviceIoControl(
      *   _In_        HANDLE       hDevice,
      *   _In_        DWORD        dwIoControlCode,
      *   _In_opt_    LPVOID       lpInBuffer,
      *   _In_        DWORD        nInBufferSize,
      *   _Out_opt_   LPVOID       lpOutBuffer,
      *   _In_        DWORD        nOutBufferSize,
      *   _Out_opt_   LPDWORD      lpBytesReturned,
      *   _Inout_opt_ LPOVERLAPPED lpOverlapped
      * );
      *
      **********************************************************/

      nLock := LockVolume( hVolRead )
      IF nLock <> 0
         cMsg := "LockVolume()" + CRLF
         SayInfo( cMsg )

         nDisMount := DismountVolume( hVolRead )
         IF nDisMount <> 0
            cMsg := "DismountVolume()" + CRLF
            SayInfo( cMsg )

            nRemove := PreventRemovalOfVolume( hVolRead, .F. )
            IF nRemove <> 0
               cMsg := "PreventRemovalOfVolume()" + CRLF
               SayInfo( cMsg )

               nEject := AutoEjectVolume( hVolRead )
               cMsg := "AutoEjectVolume()" + CRLF
               SayInfo( cMsg )

               lRet := .T.
               SayInfo( "remove your USE-Stick now" )
               hb_IdleSleep( 2.0 )
            ELSE
               cMsg := "Error : can not PreventRemovalOfVolume() " + VAR2CHAR( nRemove )
               lRet := .F.
            ENDIF
         ELSE
            cMsg := "Error : can not DismountVolume() " + VAR2CHAR( nDisMount )
            lRet := .F.
         ENDIF
      ELSE
         cMsg := "Error : can not LockVolume() " + VAR2CHAR( nLock )
         lRet := .F.
      ENDIF

   ENDIF

   // Close Handle here
   CloseVolume( hVolRead )

   IF lRet = .F.

   ELSE
      IF nType <> DRIVE_CDROM                                         // will eject
         //  MEMOWRIT( cPath + "ShowMsg.BAT", cRoot + cAction )
         //  ShellOpenFile( cPath, "ShowMsg.BAT" )
         //  SLEEP( 300 )
         //  FERASE( cPath + "ShowMsg.BAT" )
      ENDIF
   ENDIF

RETURN lRet

*+--------------------------------------------------------------------
*+
*+    Static Function CreateVolHandle()
*+
*+    Called from ( usbeject.prg )   1 - function ejectmedia()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION CreateVolHandle( cMsg, nType, nAccess, cDrive )

LOCAL hVolWrite
LOCAL hVolRead
LOCAL nFlush    := 0                                                  // "FlushFileBuffers"
LOCAL cPath     := ""
LOCAL nShare    := FILE_SHARE_READ + FILE_SHARE_WRITE
LOCAL nDispo    := OPEN_EXISTING
LOCAL nAttrib   := 0                                                  // FILE_FLAG_OVERLAPPED + FILE_FLAG_BACKUP_SEMANTICS

   /**********************************************************
   *
   * https://msdn.microsoft.com/en-us/library/aa363858.aspx
   *
   * HANDLE WINAPI CreateFile(
   *   _In_     LPCTSTR               lpFileName,
   *   _In_     DWORD                 dwDesiredAccess,
   *   _In_     DWORD                 dwShareMode,
   *   _In_opt_ LPSECURITY_ATTRIBUTES lpSecurityAttributes,
   *   _In_     DWORD                 dwCreationDisposition,
   *   _In_     DWORD                 dwFlagsAndAttributes,
   *   _In_opt_ HANDLE                hTemplateFile
   * );
   *
   **********************************************************/

   cPath := "\\.\"
   cPath += cDrive
   cPath += ":"

   // Kernel32
   hVolWrite := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "CreateFile", cPath, ;                                                              // lpFileName
                             nAccess, ;                               // dwDesiredAccess
                             nShare, ;                                // dwShareMode
                             NULL, ;                                  // lpSecurityAttributes
                             nDispo, ;                                // dwCreationDisposition
                             nAttrib, ;                               // dwFlagsAndAttributes
                             NULL )                                   // hTemplateFile

   IF hVolWrite = - 1
      RETURN hVolWrite
   ELSE
      // flush buffer
      IF nType <> DRIVE_CDROM
         // Kernel32
         nFlush := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "FlushFileBuffers", hVolWrite )
         cMsg := "FlushFile()" + CRLF
         SayInfo( cMsg )
      ENDIF
   ENDIF

   // Kernel32
   HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "CloseHandle", hVolWrite )

   // open again RO
   nAccess := GENERIC_READ
   // Kernel32
   hVolRead := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "CreateFile", cPath, ;                                                               // lpFileName
                            nAccess, ;                                // dwDesiredAccess
                            nShare, ;                                 // dwShareMode
                            NULL, ;                                   // lpSecurityAttributes
                            nDispo, ;                                 // dwCreationDisposition
                            nAttrib, ;                                // dwFlagsAndAttributes
                            NULL )                                    // hTemplateFile

RETURN hVolRead

*+--------------------------------------------------------------------
*+
*+    Static Function LockVolume()
*+
*+    Called from ( usbeject.prg )   1 - function ejectmedia()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION LockVolume( hVolRead )

LOCAL nBytes := 0
LOCAL nLock

   // Kernel32
   nLock := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "DeviceIoControl", hVolRead, ;
                         FSCTL_LOCK_VOLUME, NULL, 0, NULL, 0, @nBytes, NULL )

RETURN nLock

*+--------------------------------------------------------------------
*+
*+    Static Function DismountVolume()
*+
*+    Called from ( usbeject.prg )   1 - function ejectmedia()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION DismountVolume( hVolRead )

LOCAL nBytes    := 0
LOCAL nDisMount

   // Kernel32
   nDisMount := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "DeviceIoControl", hVolRead, ;
                             FSCTL_DISMOUNT_VOLUME, NULL, 0, NULL, 0, @nBytes, NULL )

RETURN nDisMount

*+--------------------------------------------------------------------
*+
*+    Static Function AutoEjectVolume()
*+
*+    Called from ( usbeject.prg )   1 - function ejectmedia()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION AutoEjectVolume( hVolRead )

LOCAL nBytes := 0
LOCAL nEject

   // Kernel32
   nEject := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "DeviceIoControl", hVolRead, ;
                          IOCTL_STORAGE_EJECT_MEDIA, NULL, 0, NULL, 0, @nBytes, NULL )

RETURN nEject

*+--------------------------------------------------------------------
*+
*+    Static Function UnlockVolume()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION UnlockVolume( hVolRead )

LOCAL nBytes  := 0
LOCAL nUnlock

   // Kernel32
   nUnlock := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "DeviceIoControl", hVolRead, ;
                           FSCTL_UNLOCK_VOLUME, NULL, 0, NULL, 0, @nBytes, NULL )

RETURN nUnlock

*+--------------------------------------------------------------------
*+
*+    Static Function CloseVolume()
*+
*+    Called from ( usbeject.prg )   1 - function ejectmedia()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION CloseVolume( hVolRead )

LOCAL lClose
   // Kernel32
   lClose := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "CloseHandleA", hVolRead )
RETURN lClose

*+--------------------------------------------------------------------
*+
*+    Static Function PreventRemovalOfVolume()
*+
*+    Called from ( usbeject.prg )   1 - function ejectmedia()
*+
*+--------------------------------------------------------------------
*+
STATIC FUNCTION PreventRemovalOfVolume( hVolRead, fPreventRemoval )

LOCAL nBytes  := 0
LOCAL nRemove
LOCAL pmr     := CHR( 0 )                                             // _PREVENT_MEDIA_REMOVAL():New()
LOCAL nSize   := 1                                                    // pmr:_sizeof_()

   //    pmr:PreventMediaRemoval := fPreventRemoval
   nSize := 1
   // Kernel32
   nRemove := HMG_CallDLL( "Kernel32.dll", DLL_OSAPI, "DeviceIoControl", hVolRead, ;
                           IOCTL_STORAGE_MEDIA_REMOVAL, @pmr, nSize, NULL, 0, @nBytes, NULL )

RETURN nRemove

*+--------------------------------------------------------------------
*+
*+    Static Procedure SayInfo()
*+
*+    Called from ( usbeject.prg )   6 - function ejectmedia()
*+                                   1 - static function createvolhandle()
*+
*+--------------------------------------------------------------------
*+
STATIC PROCEDURE SayInfo( cText )

   cEject += cText + CRLF

   SetProperty( "EjectDlg", "EjectInfo", "VALUE", cEject )

   DO EVENTS
   hb_IdleSleep( 0.5 )

RETURN

*+ EOF: USBEJECT.PRG
